#!/bin/sh

set -e

# BE CAREFUL!
#
# This script can execute via cclub-fix-debian-mirror-configuration, which
# does not have an equal-version dependency on cclub-pre-configuration.
# Thus, the latest and greatest function.sh features may not be available.

##functions.sh##

target="$1"
self="$2"
template="${self%.sh}"
flags="$3"

tmpfile=`mktemp --tmpdir update-apt-sources-list.XXXXXXXX`

cleanup() {
    rm -f "$tmpfile"
}

trap cleanup EXIT 

ws="[[:space:]]+"

variants='-cclub|-lts(-kernel)?|-updates|/updates'
stretch="stretch($variants)?"
repos='debian(-security|-volatile)?'
cclub_mirror="(debian|ftp|www)\\.club\\.cc\\.cmu\\.edu/(pub/)?(archive\\.debian\\.org/)?$repos/?"
mit_mirror="(debian\\.lcs|mirrors)\\.mit\\.edu/$repos/?"
fastly_cdn="cdn-fastly\\.deb\\.debian\\.org/$repos/?"
security='security\.debian\.org(/debian-security)?/?'
freexian='deb\.freexian\.com/extended-lts/?'

if is_flag_set "$flags" force; then
    :
else
    pattern="${ws}http://($cclub_mirror|$mit_mirror|$fastly_cdn|$security|$freexian)${ws}${stretch}${ws}"
    egrep "$pattern" "$target" > "$tmpfile" || true
    if egrep "$pattern" "$template" | cmp -s "$tmpfile"; then
        debug_echo -n "It appears \"$target\" is already configured for "
        debug_echo "stretch; skipping update"
        exit 0
    fi
fi

variants="$variants|-backports|-volatile"
jessie="jessie($variants)?"
wheezy="wheezy($variants)?"
squeeze="squeeze($variants)?"
lenny="lenny($variants)?"
repos='(debian(-archive|-backports|-security|-volatile)?|backports\.org)'
cclub_mirror="(debian|ftp|www)\\.club\\.cc\\.cmu\\.edu/(pub/)?(archive\\.debian\\.org/)?$repos/?"
mit_mirror="(debian\\.lcs|mirrors)\\.mit\\.edu/$repos/?"
pair_mirror="debian\\.mirrors\\.pair\\.com/$repos/?"
main="ftp\\.(us\\.)?debian\\.org/(pub/)?$repos/?"
redir="(deb|httpredir)\\.debian\\.org/$repos/?"
fastly_cdn="cdn-fastly\\.deb\\.debian\\.org/$repos/?"
archive="archive\\.debian\\.org/$repos/?"
known_url="($cclub_mirror|$security|$freexian|$mit_mirror|$pair_mirror|$main|$redir|$fastly_cdn|$archive)"

sed -r -e '# comment out lines for any old releases'                \
       -e "\\%${ws}($jessie|$wheezy|$squeeze|$lenny)${ws}%{"        \
       -e '    s%^([[:space:]]*[^#[:space:]])%#\1%'                 \
       -e '}'                                                       \
       -e '# delete comments from prior template application'       \
       -e '/^# Default cclub sources\.list for Debian/d'            \
       -e '/^# [a-z]+-updates, previously.*volatile/d'              \
       -e '# delete "regular" repos for the current release'        \
       -e "\\%${ws}(http|ftp)://${known_url}${ws}${stretch}${ws}%d" \
    "$target" > "$tmpfile"
echo >> "$tmpfile"
cat "$template" >> "$tmpfile"

# turn sequences of empty lines into a single empty line
$dry_run eval "awk -- 'BEGIN      { n = 1 }
                       /^[ \t]*$/ { if (++n <= 1) print \"\" }
                       /[^ \t]/   { n = 0; print }'            \\
                   < `quote "$tmpfile"` > `quote "$target"`"
